﻿using UnityEngine;
using System.Collections;
using Holoville.HOTween;

public class Crosshair:MonoBehaviour {
	public Color selectionColor;

	// Handle to Camera
	public Camera cam;

	// Positions
	private Vector3 startPos, endPos;
	private Vector3 tempPos;
	private RaycastHit hit;

	// Selected
	public static bool selected = false;

	// Current selection
	private Collider currentSelectionCollider = null;
	private CrosshairTarget currentSelectionTarget = null;

	// Time
	private float selectionTimer = 0f;

	// Time to select
	private const float TIME_TO_SELECT = 1.5f;
	private const float TIME_TO_SELECT_START = 3f; // TODO - revert to 3f

	// Init
	public void Awake() {
		HOTween.Init();
	}

	// Update
	public void Update() {
		if(selected && LevelManager.instance.started) {
			return;
		}

		startPos = cam.transform.position;
		endPos = startPos + (cam.transform.forward * 1000f);

		if(Physics.Linecast(startPos, endPos, out hit)) {
			if(hit.collider == currentSelectionCollider) {
				selectionTimer += Time.deltaTime;
				if(LevelManager.instance.started && selectionTimer >= TIME_TO_SELECT) {
					SelectShape();
					currentSelectionTarget.particles.startColor = Color.white;
					LevelManager.instance.audioFilter.cutoffFrequency = 22000f;
					selected = true;
				} else if(! LevelManager.instance.started && selectionTimer >= TIME_TO_SELECT_START) {
					LevelManager.instance.audioFilter.cutoffFrequency = 22000f;
					LevelManager.instance.StartPlaying();
				} else {
					SelectionUpdate();
				}
			} else {
				selectionTimer = 0f;
				currentSelectionCollider = hit.collider;
				if(currentSelectionTarget != null) {
					currentSelectionTarget.particles.transform.localScale = Vector3.one;
					currentSelectionTarget.particles.startColor = Color.white;
					LevelManager.instance.audioFilter.cutoffFrequency = 22000f;
				}
				currentSelectionTarget = currentSelectionCollider.GetComponent<CrosshairTarget>();
				currentSelectionTarget.particles.startColor = selectionColor;
			}
		} else {
			if(currentSelectionTarget != null) {
				currentSelectionTarget.particles.transform.localScale = Vector3.one;
				currentSelectionTarget.particles.startColor = Color.white;
				LevelManager.instance.audioFilter.cutoffFrequency = 22000f;
				if(!LevelManager.instance.started) {
					currentSelectionTarget.shape.GetComponent<StartText>().time = 0f;
				}
			}
			selectionTimer = 0f;
			currentSelectionCollider = null;
			currentSelectionTarget = null;
		}
	}

	public void SelectionUpdate() {
		float fl = 0f;
		if(LevelManager.instance.started) {
			fl = (1f - (selectionTimer/TIME_TO_SELECT));
			currentSelectionTarget.particles.transform.localScale = Vector3.one * fl;
			LevelManager.instance.audioFilter.cutoffFrequency = 10000f * fl;
		} else {
			fl = (1f - (selectionTimer/TIME_TO_SELECT_START));
			currentSelectionTarget.particles.transform.localScale = Vector3.one * fl;
			currentSelectionTarget.shape.GetComponent<StartText>().time = selectionTimer/TIME_TO_SELECT_START;
			LevelManager.instance.audioFilter.cutoffFrequency = 10000f * fl;
		}
	}

	public void SelectShape() {
		if(currentSelectionCollider.name == LevelManager.instance.targetShape.name) {
			// Correct!

			// Grow shape
			GameObject shape = currentSelectionCollider.GetComponent<CrosshairTarget>().shape;
			shape.transform.localScale = Vector3.zero;
			shape.layer = LayerMask.NameToLayer("IgnoredByTopCam");
			HOTween.To(shape.transform, 0.75f, new TweenParms().Prop("localScale", Vector3.one).Ease(EaseType.EaseOutBounce));
			
			// Particles explode
			currentSelectionTarget.particles.Clear();
			currentSelectionTarget.particles.transform.localScale = Vector3.one * 0.1f;
			currentSelectionTarget.particles.startSpeed = 5f;

			// Other shapes disappear
			LevelManager.instance.HideAnswerShapes(false);
		} else {
			// All shapes disappear
			LevelManager.instance.HideAnswerShapes(true);
		}
	}
}